package hydro;
/**
 * Last Revision Feb.17 2008
 * Title: NetcdfReader.java
 * Description: Convert netcdf file into dat file. 
 * Copyright: Copyright (c) 2009
 * Company: CIMAS/UM & AOML/NOAA
 * @author Chao Chen
 * @version 1.1
 */



//~--- non-JDK imports --------------------------------------------------------

/**
 *
 * @author xia
 */
import ucar.ma2.*;
import ucar.nc2.*;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.*;
import java.io.*;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

public class NetcdfReader {
	static Logger log = Logger.getLogger(NetcdfReader.class);
    Array     a;
    ArrayList dimensions;
    ArrayList content,content3;
    ArrayList cycle;
    ArrayList vars,charList,intList,doubleList,floatList;

    ArrayList pres,pres_qc,pres_adjusted,pres_adjusted_qc,pres_adjusted_error;
    ArrayList temp,temp_qc,temp_adjusted,temp_adjusted_qc,temp_adjusted_error;
    ArrayList psal,psal_qc,psal_adjusted,psal_adjusted_qc,psal_adjusted_error;;
    ArrayList doxy,doxy_qc,doxy_adjusted,doxy_adjusted_qc,doxy_adjusted_error;;      
    ArrayList juld_as,juld_as_status,juld_ae,juld_ae_status,juld_ds,juld_ds_status,juld_de,juld_de_status,juld_st,juld_st_status,grounded;
    
    File datafile,datafileAppend,datafile2=null;
    int adjust_num=0;
    FileWriter fw,fw3=null;
   
    String pcode="";
    String parm="";
    String q_parm="";
    String surfacecodes_pcode="";
    String surfacecodes_cparm="";
    String surfacecodes_qparm="";
    
    BufferedWriter bw,bw3=null;

    String outputType="";
    /** Creates a new instance of NetcdfReader */
    public NetcdfReader() {}
   
    public NetcdfReader(String fileName, String outputDir) {   
    	
    	try {
			NetcdfFile nc = new NetcdfFile(fileName);
			Iterator vit=nc.getVariableIterator();
			String mode="";
				 while(vit.hasNext()){
	   			 Variable v=(Variable)vit.next();
	   			 String name=v.getName();
	   			 if(v.getName().equalsIgnoreCase("format_version")||v.getName().equalsIgnoreCase("handbook_version")||v.getName().equalsIgnoreCase("DATA_TYPE")){
	   				a=v.read();
	   				IndexIterator iter = a.getIndexIterator();
	   				String version="";
	   				while(iter.hasNext()){
	   					char val = iter.getCharNext();
	   					version=version+val;
	   				}
	   				version=version.toLowerCase();
	   				if(version.contains("argo")){
	   					mode="ARGO";
	   				}
	   				else if(version.contains("gtspp")){
	   					mode="GTSPP";
	   				}
	   			 }
	   		 }
			 if(mode.equals("ARGO")){
				 argoReader(fileName,outputDir);
			 }else if(mode.equals("GTSPP")){			    
				 gtsppReader(fileName,outputDir);
			 }
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
    }    // end of constructor    
   
    private void gtsppReader(String fileName, String outputDir) {
		
		// TODO Auto-generated method stub
    	try {   		 
   		 NetcdfFile nc = new NetcdfFile(fileName);
   		 //initialize
   		 dimensions=new ArrayList();
   		 boolean InstOut=true;
   		 
   		 if(getFileName(fileName).contains("traj")){
   			 datafile=new File(outputDir+File.separator+getFileName(fileName)+"1.dat"); 
      		 datafile2=new File(outputDir+File.separator+getFileName(fileName)+"2.dat"); 
  		 }
   		 else{
   			 datafile=new File(outputDir+File.separator+getFileName(fileName)+".dat"); 
   		 }
   		 
            fw=new FileWriter(datafile);
            bw=new BufferedWriter(fw);
            
            
   		 content=new ArrayList();
   		 content3=new ArrayList();
   		 cycle=new ArrayList();
   		 boolean moreFile=false;
   		 boolean adjust_flag=true;
   		 boolean realdepth=true;
   		 
   		 //get the dimensions		 
   		 Iterator dit=nc.getDimensionIterator();   		 
   		 while(dit.hasNext()){
   			 Dimension d=(Dimension)dit.next();    			
   			 dimensions.add(d);
   		 }
   		 dit.remove();
   		    		 
   		 //start iterating all variables
   		 Iterator vit=nc.getVariableIterator();
   		 String date="";
   		 while(vit.hasNext()){
   			 Variable v=(Variable)vit.next();
   			 String name=v.getName();
   			    		
   			 if(name.equalsIgnoreCase("depth")){
   				Attribute depthUnit=v.findAttribute("units");
   				if(depthUnit.getStringValue().equals("decibars")){
   					realdepth=false;
   				}
   			 }
   			 if(isInGTSPPlist(name)){ 			
   			//System.out.println(name);
   			 String type=v.getElementType().toString();
   			 //output once
   			 InstOut=checkOutGtspp(v.getDimensions());
   			 
   			 //if(name.equalsIgnoreCase("DEPH_qparm")){
   			 //	 System.out.println("InstOut="+InstOut);
   			 //}
   			 //if(name.contains("ADJUSTED")){
   			 //	 adjust_num++;
   			 //}
   			
   			 a=v.read();
   			 
   			 //get printout format
   			 Attribute at=v.findAttribute("C_format");
   			 
   			 if(at!=null){
   				 outputType=at.getStringValue();
   			 }
   			 else if(name.equals("LONGITUDE")||name.equals("LATITUDE")){
   				 outputType="%9.3f";
   			 }
   			 else if(v.getElementType().equals("double")){
   				 outputType="%20.13f";
   			 }else if(v.getElementType().equals("char")){
   				 outputType="%d";
   			 }else{
   				 outputType="";
   			 }
   			     			 
   			 String CONTENT="";
   			 
   			 //get each element (single character)
   			 IndexIterator iter = a.getIndexIterator();
   			 while (iter.hasNext()) {
   				
   					if(type.equals("char")){   						
   						char val = iter.getCharNext();   
   								if(InstOut==true&&!(name.equals("obs_year")||name.equals("obs_month")||name.equals("obs_day")||name.equals("obs_hour")||name.equals("obs_minute"))){
            						CONTENT=CONTENT+val;
            					} 
            					else if(name.equals("obs_year")||name.equals("obs_month")||name.equals("obs_day")||name.equals("obs_hour")){             						
            						date=date+val;   
            						//System.out.println("date="+date);
            					}else if(name.equals("obs_minute")){
            						date=date+val;
            						CONTENT=date;  
            						//System.out.println("content="+CONTENT);
            						InstOut=true;
            					}
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}   
   					
   					if(type.equals("int")){ 
   						int val = iter.getIntNext();                                           
            					if(InstOut==true){
            						CONTENT=CONTENT+val;
            					} 
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}   
   					if(type.equals("float")){    						
   						float val = iter.getFloatNext();                                           
            					if(InstOut==true){
            						CONTENT=CONTENT+val;
            					} 
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}  
   					if(type.equals("double")){ 
   						double val = iter.getDoubleNext();   
   							
   							
   							if(InstOut==true){
            						CONTENT=CONTENT+val;
            					} 
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}  
   			 }
   			 
   			 if(InstOut==true){

   				if(name.equals("obs_year")||name.equals("obs_month")||name.equals("obs_day")||name.equals("obs_hour")){}
   				else if(name.equals("obs_minute")){
   					CONTENT = CONTENT.trim(); 
   					name=padName("DATE/TIME [YYYY MM DD HH MM]"); 
   					bw.append(name+DateFMT(CONTENT));
   					bw.append("\n");
   				 }
   				 else if(name.equals("q_record")){
   					CONTENT = CONTENT.trim(); 
   					name=padName("PROFILE_QC"); 
   					bw.append(name+CONTENT);
   					bw.append("\n");
   				 }
   				 else if(name.equals("up_date")){
   					CONTENT = CONTENT.trim(); 
   					name=padName("DATE_UPDATE"); 
   					bw.append(name+CONTENT);
   					bw.append("\n");
   				 }
   				 else{
   					 CONTENT = CONTENT.trim(); 
            		 name=padName(name.toUpperCase()); 
        			 bw.append(name+CONTENT);
        			 bw.append("\n");
   				 }
   			 }else if(name.contains("pcode")||name.contains("parm")){

   				CONTENT=CONTENT.substring(0, CONTENT.length()-1); //remove the last comma.   
   				if(name.equals("pcode")){
   					pcode=CONTENT;
   				}
   				if(name.equals("parm")){
   					parm=CONTENT;
   				}
   				if(name.equals("q_parm")){
   					q_parm=CONTENT;
   				}
   				if(name.equals("surfacecodes_pcode")){
   					surfacecodes_pcode=CONTENT;
   				}
   				if(name.equals("surfacecodes_cparm")){
   					surfacecodes_cparm=CONTENT;
   				}
   				if(name.equals("surfacecodes_qparm")){
   					surfacecodes_qparm=CONTENT;
   				}
   				if(name.equalsIgnoreCase("DEPH_qparm")||name.equalsIgnoreCase("TEMP_qparm")||name.equalsIgnoreCase("PSAL_qparm")){
   					
   	   				if(name.equalsIgnoreCase("DEPH_qparm")){
   	   					if(realdepth==true){
   	   						name="DEPTH_QC";
   	   					}
   	   					else{
   	   						name="PRES_QC";
   	   					}
   	   				}
   	   				if(name.equalsIgnoreCase("TEMP_qparm")){
   	   					name="TEMP_QC";
   	   				}
   	   				if(name.equalsIgnoreCase("PSAL_qparm")){
   	   					name="PSAL_QC";
   	   				}   	
   					content.add(new NetcdfData(name.toUpperCase(),CONTENT,outputType));  
   				}
   			 }
   			 else{   			  
   				CONTENT=CONTENT.substring(0, CONTENT.length()-1); //remove the last comma.   				
   				content.add(new NetcdfData(name.toUpperCase(),CONTENT,outputType));   
   			 }
   			 }
   		 }	
   		 //System.out.println(content.size());
   		 //pcode parm and q_parm
   		 String[] tmpPar3=pcode.split(",");
   		 String[] param3=new String[tmpPar3.length/4];
   		 //initialize to "";
   		 for(int i=0;i<param3.length;i++){   				
			param3[i]="";
	     }   		 
   		 for(int i=0;i<tmpPar3.length;i++){   				
				int Pindex=i/4;
				param3[Pindex]+=tmpPar3[i];
		 }
   		 for(int i=0;i<param3.length;i++){   				
			param3[i]=padName(param3[i]);
   		 }
   		 tmpPar3=parm.split(",");
   		 
   		 for(int i=0;i<tmpPar3.length;i++){   				
   				int Pindex=i/10;
   				param3[Pindex]+=tmpPar3[i];
   		 }
   		 
   		 tmpPar3=q_parm.split(",");
   		 for(int i=0;i<tmpPar3.length;i++){
 			 param3[i]+="  "+tmpPar3[i];
 		 }
   		 
   		    		 
   		 if(!containBlank(pcode)){
   		 //bw.write(padName("pcode")+"      parm"+"  "+"q_parm\n");
   		 bw.write(padName("PCODE")+"PARM"+"  "+"Q_PARM\n");
   		 for(int i=0;i<param3.length;i++){
			 bw.write(param3[i]+"\n");
		 }
   		 }
   		 
   		 //surfacecodes_pcode surfacecodes_cparm and surfacecodes_qparm
   		 tmpPar3=surfacecodes_pcode.split(",");
   		 param3=new String[tmpPar3.length/4];
   		 //initialize to "";
   		 for(int i=0;i<param3.length;i++){   				
			param3[i]="";
	     }   		 
   		 for(int i=0;i<tmpPar3.length;i++){   				
				int Pindex=i/4;
				param3[Pindex]+=tmpPar3[i];
		 }
   		 for(int i=0;i<param3.length;i++){   				
			param3[i]=padName(param3[i]);
   		 }
   		 tmpPar3=surfacecodes_cparm.split(",");
   		 
   		 for(int i=0;i<tmpPar3.length;i++){   				
   				int Pindex=i/10;
   				param3[Pindex]+=tmpPar3[i];
   		 }
   		 
   		 tmpPar3=surfacecodes_qparm.split(",");
   		 for(int i=0;i<tmpPar3.length;i++){
 			 param3[i]+="  "+tmpPar3[i];
 		 }
   		 //bw.write(padName("pcode")+"      parm"+"  "+"q_parm\n");
   		
   		 if(!containBlank(surfacecodes_pcode)){
   		 bw.write(padName("SURFACECODES_PCODE")+"SURFACECODES_CPARM"+"  "+"SURFACECODES_QPARM\n");
   		 for(int i=0;i<param3.length;i++){
			 bw.write(param3[i]+"\n");
		 }
   		 }
   		 
   		 
   		 bw.write(padName("NUMBER OF COLUMNS"));  
   		 bw.write(content.size()+"\n");   		 
   		 bw.write(padName("COLUMN 1"));
   		 NetcdfData ncd=(NetcdfData)content.get(0);
			 String[] content_data=ncd.getData().split(",");
			 for(int i=0;i<content_data.length;i++){
				 if(!(ncd.getOutputType().equals(""))){					 
					 content_data[i]=new PrintfFormat(ncd.getOutputType()).sprintf(Double.parseDouble(content_data[i]));
				 }
			 }
			 //System.out.println(ncd.getName());
			 if(ncd.getName().equalsIgnoreCase("depth")&&(realdepth==false)){
				 bw.write("PRES\n");
			 }else if(ncd.getName().equalsIgnoreCase("temperature")){
				 bw.write("TEMP\n");
			 }else if(ncd.getName().equalsIgnoreCase("salinity")){
				 bw.write("PSAL\n");
			 }
			 else{
				 bw.write(ncd.getName()+"\n");
			 }
			 int index=1;
   		 for(int i=1;i<content.size();i++){   	   			 
   			 NetcdfData ncdtmp=(NetcdfData)content.get(i);
   			 if(!(adjust_flag==false&&ncdtmp.getName().contains("ADJUSTED"))){
   			 String[] tmp=ncdtmp.getData().split(",");    			 
   			 for(int j=0;j<tmp.length;j++){
   				 
   				 if(!ncdtmp.getOutputType().equals("")){
   					 content_data[j]=content_data[j]+" "+new PrintfFormat(ncdtmp.getOutputType()).sprintf(Double.parseDouble(tmp[j]));
   				 }else{
   					 content_data[j]=content_data[j]+" "+tmp[j];
   				 }
   			 }    			 
   				 bw.write(padName("COLUMN "+(index+1)));
   				 if(ncdtmp.getName().equalsIgnoreCase("depth")&&(realdepth==false)){
   					 bw.write("PRES\n");
   				 }else if(ncdtmp.getName().equalsIgnoreCase("temperature")){
   					 bw.write("TEMP\n");
   				 }else if(ncdtmp.getName().equalsIgnoreCase("salinity")){
   					 bw.write("PSAL\n");
   				 }
   				 else{
   					 bw.write(ncdtmp.getName()+"\n");
   				 }
   				 //bw.write(ncdtmp.getName().toUpperCase()+"\n");
   				 index++;
   			 }
   		 }
   		 bw.write("=============================================================\n");
   		 for(int j=0;j<content_data.length;j++){
				 bw.write(content_data[j]+"\n");
   		 }
   		 if(moreFile==true){
   			 
   			 fw3=new FileWriter(datafile2);
   			 bw3=new BufferedWriter(fw3);
   			 bw3.write(padName("NUMBER OF COLUMNS"));
       		 
       		 bw3.write(cycle.size()+"\n");
       		 bw3.write(padName("COLUMN 1"));
       		 NetcdfData ncdcyc=(NetcdfData)cycle.get(0);
   			 String[] cycle_data=ncdcyc.getData().split(",");
   			 for(int i=0;i<cycle_data.length;i++){
   				 if(!ncdcyc.getOutputType().equals("")){
   					 content_data[i]=content_data[i]+" "+new PrintfFormat(ncdcyc.getOutputType()).sprintf(Double.parseDouble(cycle_data[i]));
   				 }else{
   					 content_data[i]=content_data[i]+" "+cycle_data[i];
   				 }
   			 }
   			 bw3.write(ncdcyc.getName().toUpperCase()+"\n");		 
   			 
       		 for(int i=1;i<cycle.size();i++){
       			
       			 bw3.write(padName("COLUMN "+(i+1)));
       			 NetcdfData ncdtmp=(NetcdfData)cycle.get(i);
       			 
       			 String[] tmp=ncdtmp.getData().split(",");    			 
       			 for(int j=0;j<tmp.length;j++){
       				 if(!ncdtmp.getOutputType().equals("")){
       					 cycle_data[j]=cycle_data[j]+" "+new PrintfFormat(ncdtmp.getOutputType()).sprintf(Double.parseDouble(tmp[j]));
       				 }else{
       					 cycle_data[j]=cycle_data[j]+" "+tmp[j];
       				 }
       			 }
       			 bw3.write(ncdtmp.getName().toUpperCase()+"\n");
       		 }
       		 bw3.write("=============================================================\n");
       		 for(int j=0;j<cycle_data.length;j++){
   				 bw3.write(cycle_data[j]+"\n");
       		 }
   		 }
   		 if(bw!=null){
   		 	bw.close();
   		 }
   		 if(fw!=null){
				fw.close();
   		 }
   		 
   		 if(bw3!=null){
				bw3.close();
			 }
			 if(fw3!=null){
				fw3.close();
			 }
   		 
   		 //System.out.println(content.size());
   	} catch (java.io.IOException e) {
           e.printStackTrace();  
           log.error("IO Exception found", e);
       } catch (Exception ex) {
           //System.out.println(ex.getMessage());
           ex.printStackTrace();
           log.error("Exception found", ex);
       }
       
       finally{
       	try {
				bw.close();
				fw.close();
				
				if(bw3!=null){
				bw3.close();
				}
				if(fw3!=null){
				fw3.close();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				log.error("IO Exception found", e);
			}
       	
       	
       }
       
	
	}

	private boolean containBlank(String pcode) {
		// TODO Auto-generated method stub
		if(pcode.trim().length()<pcode.length()){
			return true;
		}
		else{
			return false;
		}
	}

	private String DateFMT(String DATETIME) {
		// TODO Auto-generated method stub
		String date="";
		for(int i=0;i<DATETIME.length();i++){
			date+=DATETIME.charAt(i);
			if(i==3||i==5||i==7||i==9){
				date+=" ";
			}
		}
		return date;
	}

	private void argoReader(String fileName,String outputDir) {
		// TODO Auto-generated method stub
    	try {   		 
   		 NetcdfFile nc = new NetcdfFile(fileName);
   		 //initialize
   		 dimensions=new ArrayList();
   		 int InstOut=0;
   		 if(getFileName(fileName).contains("traj")){
  			 datafile=new File(outputDir+File.separator+getFileName(fileName)+"1.dat"); 
     		 datafile2=new File(outputDir+File.separator+getFileName(fileName)+"2.dat"); 
 		 }
  		 else{
  			 datafile=new File(outputDir+File.separator+getFileName(fileName)+".dat"); 
  		 }
   		 
            fw=new FileWriter(datafile);
            bw=new BufferedWriter(fw);

            
   		 content=new ArrayList();
   		 cycle=new ArrayList();
   		 boolean moreFile=false;
   		 boolean adjust_flag=true;
   		 
   		 //get the dimensions		 
   		 Iterator dit=nc.getDimensionIterator();   		 
   		 while(dit.hasNext()){
   			 Dimension d=(Dimension)dit.next();    			
   			 dimensions.add(d);
   		 }
   		 dit.remove();
   		    		 
   		 Iterator vit=nc.getVariableIterator();
   		 while(vit.hasNext()){
   			 Variable v=(Variable)vit.next();
   			 String name=v.getName();
   			 
   			 
   			 if(isInlist(name)){ 			
   			//System.out.println(name);
   			 String type=v.getElementType().toString();
   			 InstOut=checkOut(v.getDimensions());
   			 
   			 if(name.contains("ADJUSTED")){
   				 adjust_num++;
   			 }
   			 //System.out.println(name);
   			 a=v.read();
   			 //System.out.println(name);
   			
   			 //get printout format
   			 Attribute at=v.findAttribute("C_format");
   			 
   			 if(at!=null){
   				 outputType=at.getStringValue();
   			 }
   			 else if(name.equals("LONGITUDE")||name.equals("LATITUDE")){
   				 outputType="%9.3f";
   			 }
   			 else if(v.getElementType().equals("double")){
   				 outputType="%20.13f";
   			 }else if(v.getElementType().equals("char")){
   				 outputType="%d";
   			 }else{
   				 outputType="";
   			 }
   			     			 
   			 String CONTENT="";
   			 
   			 
   			 IndexIterator iter = a.getIndexIterator();
   			 while (iter.hasNext()) {
   					if(type.equals("char")){ 
   						char val = iter.getCharNext();   
   							if(name.equals("DATA_MODE")&&!getFileName(fileName).contains("traj")&&val=='R'){
   								adjust_flag=false;
   							}
            					if(InstOut==0){
            						CONTENT=CONTENT+val;
            					} 
            					else if(name.equals("DATA_MODE")||name.equals("GROUNDED")){             						
            						CONTENT=CONTENT+(int)val+",";             						
            					}else if(name.endsWith("QC")&&val==' '){
            						CONTENT=CONTENT+"0"+",";
            					}else if(name.equals("POSITION_ACCURACY")){
            						if(val==' '){
            							CONTENT=CONTENT+"9"+",";
            						}
            						else if(val=='A'||val=='B'||val=='Z'||val=='G'){
            							CONTENT=CONTENT+(int)val+",";
            						} 
            						else{
            							CONTENT=CONTENT+val+",";
            						}
            					}else if(name.startsWith("JULD")&&(name.endsWith("STATUS"))){
            						if(val==' '){
            							CONTENT=CONTENT+"9"+",";
            						}
            						else{
            							CONTENT=CONTENT+val+",";
            						}
            							
            					}
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}   
   					if(type.equals("int")){ 
   						int val = iter.getIntNext();                                           
            					if(InstOut==0){
            						CONTENT=CONTENT+val;
            					} 
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}   
   					if(type.equals("float")){ 
   						float val = iter.getFloatNext();                                           
            					if(InstOut==0){
            						CONTENT=CONTENT+val;
            					} 
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}  
   					if(type.equals("double")){ 
   						double val = iter.getDoubleNext();   
   							
   							//if(name.equals("JULD_ASCENT_START")){
   							//	System.out.println(val);
   							//}
   							if(InstOut==0){
            						CONTENT=CONTENT+val;
            					} 
            					else{
            						CONTENT=CONTENT+val+",";
            					}
   					}  
   			 }
   			 if(InstOut==0){
   				CONTENT = CONTENT.trim(); 
            		name=padName(name); 
        			bw.append(name+CONTENT);
        			bw.append("\n");
        			//System.out.println(name);
   			 }else if(InstOut==1){
   				CONTENT=CONTENT.substring(0, CONTENT.length()-1); //remove the last comma.
   				
   				content.add(new NetcdfData(name,CONTENT,outputType));   
   				//System.out.println(name);
   			 }else if(InstOut==2){
   				moreFile=true;
   				CONTENT=CONTENT.substring(0, CONTENT.length()-1); //remove the last comma.
   				
    				cycle.add(new NetcdfData(name,CONTENT,outputType)); 
   			 }
   			 }
   		 }		 
   		 
   		 
   		 bw.write(padName("NUMBER OF COLUMNS"));  
   		 if(adjust_flag==false){
   			 bw.write((content.size()-adjust_num)+"\n");
   		 }else{
   			 bw.write(content.size()+"\n");
   		 }
   		 bw.write(padName("COLUMN 1"));
   		 NetcdfData ncd=(NetcdfData)content.get(0);
			 String[] content_data=ncd.getData().split(",");
			 for(int i=0;i<content_data.length;i++){
				 if(!ncd.getOutputType().equals("")){					 
					 content_data[i]=new PrintfFormat(ncd.getOutputType()).sprintf(Double.parseDouble(content_data[i]));
				 }
			 }
			 bw.write(ncd.getName()+"\n");		 
			 int index=1;
   		 for(int i=1;i<content.size();i++){   	   			 
   			 NetcdfData ncdtmp=(NetcdfData)content.get(i);
   			 if(!(adjust_flag==false&&ncdtmp.getName().contains("ADJUSTED"))){
   			 String[] tmp=ncdtmp.getData().split(",");    			 
   			 for(int j=0;j<tmp.length;j++){
   				 
   				 if(!ncdtmp.getOutputType().equals("")){
   					 content_data[j]=content_data[j]+" "+new PrintfFormat(ncdtmp.getOutputType()).sprintf(Double.parseDouble(tmp[j]));
   				 }else{
   					 content_data[j]=content_data[j]+" "+tmp[j];
   				 }
   			 }    			 
   				 bw.write(padName("COLUMN "+(index+1)));
   				 bw.write(ncdtmp.getName().toUpperCase()+"\n");
   				 index++;
   			 }
   		 }
   		 bw.write("=============================================================\n");
   		 for(int j=0;j<content_data.length;j++){
				 bw.write(content_data[j]+"\n");
   		 }
   		 if(moreFile==true){
             
             fw3=new FileWriter(datafile2);
             bw3=new BufferedWriter(fw3);
   			 bw3.write(padName("NUMBER OF COLUMNS"));
       		 
       		 bw3.write(cycle.size()+"\n");
       		 bw3.write(padName("COLUMN 1"));
       		 NetcdfData ncdcyc=(NetcdfData)cycle.get(0);
   			 String[] cycle_data=ncdcyc.getData().split(",");
   			 //System.out.println(cycle_data.length);
   			 //for(int i=0;i<cycle_data.length;i++){
   			//	 if(!(ncdcyc.getOutputType().equals(""))){
   			//		cycle_data[i]=cycle_data[i]+" "+new PrintfFormat(ncdcyc.getOutputType()).sprintf(Double.parseDouble(cycle_data[i]));
   			//	 }else{
   			//		cycle_data[i]=cycle_data[i]+" "+cycle_data[i];
   			//	 }
   			// }
   			
   			 //System.out.println(ncdcyc.getName());
   			 bw3.write(ncdcyc.getName()+"\n");		 
   			 
       		 for(int i=1;i<cycle.size();i++){
       			
       			 bw3.write(padName("COLUMN "+(i+1)));
       			 NetcdfData ncdtmp=(NetcdfData)cycle.get(i);
       			 
       			 String[] tmp=ncdtmp.getData().split(",");    			 
       			 for(int j=0;j<tmp.length;j++){
       				 if(!ncdtmp.getOutputType().equals("")){
       					 cycle_data[j]=cycle_data[j]+" "+new PrintfFormat(ncdtmp.getOutputType()).sprintf(Double.parseDouble(tmp[j]));
       				 }else{
       					 cycle_data[j]=cycle_data[j]+" "+tmp[j];
       				 }
       			 }
       			 bw3.write(ncdtmp.getName().toUpperCase()+"\n");
       		 }
       		 bw3.write("=============================================================\n");
       		 for(int j=0;j<cycle_data.length;j++){
   				 bw3.write(cycle_data[j]+"\n");
       		 }
   		 }
   		 if(bw!=null){
   		 	bw.close();
   		 }
   		 if(fw!=null){
				fw.close();
   		 }
   		 
   		 if(bw3!=null){
				bw3.close();
			 }
			 if(fw3!=null){
				fw3.close();
			 }
   		 
   		 //System.out.println(content.size());
   	} catch (java.io.IOException e) {
           e.printStackTrace();  
           log.error("IO Exception found", e);
       } catch (Exception ex) {
           //System.out.println(ex.getMessage());
           ex.printStackTrace();
           log.error("Exception found", ex);
       }
       
       finally{
       	try {
				bw.close();
				fw.close();
				
				if(bw3!=null){
				bw3.close();
				}
				if(fw3!=null){
				fw3.close();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				log.error("IO Exception found", e);
			}
       	
       	
       }
       
	}

	private String[][] getDataFromContent(ArrayList content) {
		// TODO Auto-generated method stub
		String [][] s=new String[content.size()][];
    	for(int i=0;i<s.length;i++){
    		NetcdfData ncd=(NetcdfData)content.get(i);
    		s[i]=ncd.getData().split(",");
    	}
		return s;
	}

	private boolean isInlist(String name) {
		// TODO Auto-generated method stub
    	boolean isInlist=false;
    	MODReadXML mrx=new MODReadXML();
		ArrayList vars=mrx.readXML("variables.xml");
	    	for(int i=0;i<vars.size();i++){
	    		XMLVariable nameInList=(XMLVariable)vars.get(i);
	    		if(name.equals(nameInList.getName())){
	    			isInlist=true;
	    		}
	    	}	    
		return isInlist;
	}
	
	private boolean isInGTSPPlist(String name) {
		// TODO Auto-generated method stub
    	boolean isInlist=false;
    	MODReadXML mrx=new MODReadXML();
		ArrayList vars=mrx.readXML("gtsppvariables.xml");
	    	for(int i=0;i<vars.size();i++){
	    		XMLVariable nameInList=(XMLVariable)vars.get(i);
	    		if(name.equals(nameInList.getName())){
	    			isInlist=true;
	    		}
	    	}	    
		return isInlist;
	}

	private int checkOut(ArrayList dimensions) {
		// TODO Auto-generated method stub
		int status=0;
		for(int i=0;i<dimensions.size();i++){
			Dimension d=(Dimension)dimensions.get(i);
			if(d.getName().equals("N_MEASUREMENT")||d.getName().equals("N_LEVELS")){
				status=1;
			}
			else if(d.getName().equals("N_CYCLE")){
				status=2;
			}
		}
    	return status;
	}
	
	private boolean checkOutGtspp(ArrayList dimensions) {
		// TODO Auto-generated method stub
		boolean status=true;
		for(int i=0;i<dimensions.size();i++){
			Dimension d=(Dimension)dimensions.get(i);
			if(d.getName().equalsIgnoreCase("depth")||d.getName().equalsIgnoreCase("no_parms")||d.getName().equalsIgnoreCase("no_surf")){
				status=false;
			}			
		}
    	return status;
	}
	
	public static void init(){
		PropertyConfigurator.configure("log4j.properties");
		log.setLevel(Level.INFO);
		
	}

	private String getFileName(String filename) {
		int index=filename.indexOf(File.separator);
		while(index!=-1){			
			filename=filename.substring(index+1);
			index=filename.indexOf(File.separator);
		}
		int dotInd=filename.indexOf(".");
		filename=filename.substring(0, dotInd);
		return filename;
	}
    
    private String padName(String name){
    	String pad=" ";
    	if (name.length()<=40&&name.length()>0){    		
    		int padlength=40-name.length();
    		for(int index=0;index<padlength;index++){
    			name+=pad;
    		}
    	}
        return name;
    } 
    
    private static String getCurrentTime() {
		// TODO Auto-generated method stub
		Calendar rightNow = Calendar.getInstance();
	    java.util.Date d=rightNow.getTime();
		Format formatter = new SimpleDateFormat ( "yyyy-MM-dd-HH-mm-ss" ) ; 
		String filename= "Timestamp: "+formatter.format (d);
		return filename;
	}
    
    public static void main(String[] args) {
         //NetcdfReaderToDatMod nr = new NetcdfReaderToDatMod("/h1/chen/ARGO/R49070_172.nc"); //without DOXY MODE=R 
    	 //NetcdfReaderToDatMod nr = new NetcdfReaderToDatMod("/h1/chen/ARGO/R4900476_001.nc"); //with DOXY MODE=R
         //NetcdfReaderToDatMod nr = new NetcdfReaderToDatMod("/h1/chen/ARGO/D4900702_001.nc"); //without DOXY MODE=D
    	// NetcdfReader nr = new NetcdfReader("/h1/chen/ARGO/R1900431_001.nc");//without DOXY MODE=A
    	//read files to be converted from txt file
    	init();
    	log.info("======  "+getCurrentTime()+ "   =====");
    	log.info("======               Begin               =====");
    	//System.out.println("======  "+getCurrentTime()+ "   =====");
    	//System.out.println("======               Begin               =====");
    	//File file=new File("/h1/chen/ARGO/fileList.txt");
    	File file=new File(args[0]);
    	try {
			FileReader fr=new FileReader(file);
			BufferedReader br=new BufferedReader(fr);
			String str="";
			String outputdir="";
			ArrayList fileList=new ArrayList();
			boolean r=false;
			while((str=br.readLine())!=null){
				if(str.startsWith("OutputDirectory")){
					outputdir=br.readLine().trim();
				}				
				if(r==true){
					fileList.add(str.trim());
				}
				if(str.startsWith("==")){
					r=true;
				}
			}
			br.close();
			fr.close();	
			if(fileList.size()==0){
				System.out.println("No files are specified");
				log.info("No files are specified");
				System.exit(0);
			}
			
			
	        
	       	String tmp="";
			//Start converting
			for(int i=0;i<fileList.size();i++){
				String filename=(String)fileList.get(i);				
				NetcdfReader nc=new NetcdfReader(filename,outputdir);
				 if(!nc.getFileName(filename).contains("traj")){
					tmp=outputdir+File.separator+nc.getFileName(filename)+".dat has been written";
	    			System.out.println(tmp);
	    			log.info(tmp);
				 }
				 else{
					tmp=outputdir+File.separator+nc.getFileName(filename)+"1.dat has been written";
					System.out.println(tmp);
					log.info(tmp);
					tmp=outputdir+File.separator+nc.getFileName(filename)+"2.dat has been written";
					System.out.println(tmp);
					log.info(tmp);					
				 }
			}		 
			//System.out.println("======             Finished              =====");
			log.info("======             Finished              =====\n\n");
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			log.error("File not found", e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			log.error("IO Exception found", e);
		}
    	
    }
}
